part of itwo_util;

extension IterableExtNull<E> on Iterable<E>? {
  bool get isNullOrEmpty => this == null || this?.length == 0;
}

extension IterableExt<E> on Iterable<E> {
  Map<T, List<E>> groupBy<T>(T f(E e)) {
    Map<T, List<E>> mapResult = Map<T, List<E>>();
    forEach((element) {
      T t = f(element);
      var list = mapResult[t] ?? <E>[];
      list.add(element);
      mapResult[t] = list;
    });
    return mapResult;
  }

  bool containsByCondition<T>(T t,f(E e)){
    return true;
  }

  Map<K, V> toMap<K, V>(K keySelector(E e), V valueTransform(E e)) {
    Map<K, V> result = {};
    forEach((element) {
      K k = keySelector(element);
      if (result.containsKey(k)) {
        throw Exception("Key exist");
      }
      V v = valueTransform(element);
      result[k] = v;
    });
    return result;
  }

  E? elementAtOrNull(int index) {
    if (index < 0 || length <= index) return null;
    return elementAt(index);
  }

  E? firstOrNull({bool f(E e)?}) {
    if(f==null)return this.isEmpty ? null : first;
    for (var value in this) {
      if (f(value)) return value;
    }
    return null;
  }

  E? firstOrNullUnconditional() => this.isEmpty ? null : first;

  E? lastOrNull({bool f(E e)?}) {
    if(f==null)return this.isEmpty ? null : last;
    for (int i = this.length - 1; i >= 0; i--) {
      E ee = this.elementAt(i);
      if (f(ee)) return ee;
    }
    return null;
  }

  E? lastOrNullUnconditional() => this.isEmpty ? null : last;

  List<E> filter(bool f(E e)) {
    var result = <E>[];
    for (var value in this) {
      if (f(value)) result.add(value);
    }
    return result;
  }

  void forIndex(f(E e, int index)) {
    for (int i = 0; i < this.length; i++) {
      f(elementAt(i), i);
    }
  }

  E? maxBy<R extends Comparable>(R f(E e)) {
    if (this.isEmpty) return null;
    E result = elementAt(0);
    R max = f(result);
    forEach((element) {
      R r = f(element);
      int i = max.compareTo(r);
      if (i < 0) {
        max = r;
        result = element;
      }
    });
    return result;
  }

  E? minBy<R extends Comparable>(R f(E e)) {
    if (this.isEmpty) return null;
    E result = elementAt(0);
    R max = f(result);
    forEach((element) {
      R r = f(element);
      int i = max.compareTo(r);
      if (i > 0) {
        max = r;
        result = element;
      }
    });
    return result;
  }

  num sumBy(num f(E e)) {
    num r = 0;
    this.forEach((element) => r += f(element));
    return r;
  }

  List<T> mapIndex<T>(T f(int index, E e)) {
    List<T> result = <T>[];
    this.forIndex((element, index) {
      T t = f(index, element);
      result.add(t);
    });
    return result;
  }

  String joinTo(String f(E e), {String separator = ","}) {
    var buffer = StringBuffer();
    for (int i = 0; i < this.length; i++) {
      var element = this.elementAt(i);
      buffer.write(f(element));
      if (i != this.length - 1) {
        buffer.write(separator);
      }
    }
    return buffer.toString();
  }
}

extension MapExt<K, V> on Map<K, V>? {
  bool containsAll(Map target) {
    if(isNullOrEmpty) return false;
    var result = true;
    target.forEach((targetKey, targetValue) {
      if (!this!.containsKey(targetKey) || this![targetKey] != targetValue) {
        result = false;
        return;
      }
    });
    return result;
  }
  bool get isNullOrEmpty => this == null || this?.length == 0;
}

ReturnType run<ReturnType>(ReturnType Function() operation) {
  return operation();
}

extension ScopeFunctionsForObject<T extends dynamic> on dynamic {
  ReturnType let<ReturnType>(ReturnType Function(T it) operationFor) {
    return operationFor(this);
  }

  T also(void Function(T it) operationFor) {
    operationFor(this);
    return this;
  }
}
