import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:itwo_flutter_base/util/util.dart';
import 'package:package_info/package_info.dart';
import 'package:url_launcher/url_launcher.dart';

class MapUtil {
  ///   IOS 需要在 info.plist 文件中添加白名单
  ///     <key>LSApplicationQueriesSchemes</key>
  ///     <array>
  ///         <string>iosamap</string>
  ///         <string>qqmap</string>
  ///         <string>baidumap</string>
  ///     </array>
  static Future<String?> openMapByAddress(BuildContext context, String address) async {
    var packageInfo = await DeviceUtil.packageInfo;
    String? urlGaode = await _canGaodde(packageInfo, address);
    String? urlBaidu = await _canBaidu(packageInfo, address);
    String? urlApple = await _canApple(packageInfo, address);
    if (urlGaode == null && urlBaidu == null && urlApple == null) {
      var url = "https://ditu.amap.com/search?query=$address";
      if (Platform.isIOS) {
        url = Uri.encodeFull(url);
      }
      await launch(url);
      return "h5";
    }
    List<Widget> list = <Widget>[];
    String? action = "";
    if (urlGaode != null) {
      list.add(CupertinoActionSheetAction(child: Text("高德地图"), onPressed: () async => action = await _toMap(context, urlGaode, "amap")));
    }
    if (urlBaidu != null) {
      list.add(CupertinoActionSheetAction(child: Text("百度地图"), onPressed: () async => action = await _toMap(context, urlBaidu, "baidu")));
    }
    if (urlApple != null) {
      list.add(CupertinoActionSheetAction(child: Text("苹果地图"), onPressed: () async => action = await _toMap(context, urlApple, "apple")));
    }
    await showCupertinoModalPopup(
        context: context,
        builder: (c) => CupertinoActionSheet(
              title: Text("选择地图", style: TextStyle(fontSize: 18)),
              cancelButton: CupertinoActionSheetAction(
                  child: Text("取消", style: TextStyle(color: Colors.red)),
                  onPressed: () {
                    action = "esc";
                    Navigator.pop(context);
                  }),
              actions: list,
            ));
    return action;
  }

  static Future<String?> _canGaodde(PackageInfo packageInfo, String address) async {
    var url = "";
    if (Platform.isAndroid) {
      url = "androidamap://poi?sourceApplication=${packageInfo.appName}&keywords=$address&dev=0";
    }
    if (Platform.isIOS) {
      url = "iosamap://poi?sourceApplication=${packageInfo.appName}&name=$address&dev=0";
      url = Uri.encodeFull(url);
    }
    var bool = await canLaunch(url);
    return bool ? url : null;
  }

  static Future<String?> _canBaidu(PackageInfo packageInfo, String address) async {
    var url = "";
    if (Platform.isAndroid) {
      url = "baidumap://map/geocoder?src=${packageInfo.packageName}&address=$address";
    }
    if (Platform.isIOS) {
      url = "baidumap://map/geocoder?src=${packageInfo.packageName}&address=$address";
      url = Uri.encodeFull(url);
    }
    var bool = await canLaunch(url);
    return bool ? url : null;
  }

  static Future<String?> _canApple(PackageInfo packageInfo, String address) async {
    String? url = "";
    if (Platform.isIOS) {
      url = "https://maps.apple.com/?address=$address";
      url = Uri.encodeFull(url);
    }
    var bool = url.isNotEmpty && await canLaunch(url);
    return bool ? url : null;
  }

  static Future<String?> _toMap(BuildContext context, String url, String action) async {
    try {
      await launch(url);
      Navigator.pop(context);
      return action;
    } catch (e) {
      print(e);
    }
    Navigator.pop(context);
    return null;
  }
}
