import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

/// 常用的自定义的 widget

/// CheckBox ，可设置图片，控制大小
class CustomCheckBox extends StatelessWidget {
  final Widget iconChecked; //选中时的 widget
  final Widget iconNormal; //未选中时的 widget
  // Widget iconDisable; //不允许点击时的 widget
  final Function(bool)? onChanged; //点击监听
  final Size size; // checkBox 的大小
  bool value = false; // 选中状态
  bool clickable = true; //是否允许点击
  double padding = 0; // icon 和 checkBox 的边距

  CustomCheckBox({
    Key? key,
    required this.iconChecked,
    required this.iconNormal,
    /* this.iconDisable,*/
    required this.size,
    required this.value,
    required this.clickable,
    this.onChanged,
    this.padding = 0,
  }) : super(key: key);

  CustomCheckBox.asset(String nameCheck, String nameNormal, double size, {Function(bool)? onChanged, double? padding, bool? value, bool? enable})
      : iconChecked = Image.asset(nameCheck),
        iconNormal = Image.asset(nameNormal),
        size = Size(size, size),
        // iconDisable = nameDisable == null ? null : Image.asset(nameDisable),
        onChanged = onChanged ?? onChanged,
        value = value ?? false,
        clickable = enable ?? true,
        padding = padding ?? 0;

  CustomCheckBox.icon(IconData checked, IconData normal, double size, {Function(bool)? onChanged, double? padding, bool? value, bool? enable})
      : iconChecked = Icon(checked),
        iconNormal = Icon(normal),
        size = Size(size, size),
        // iconDisable = nameDisable == null ? null : Image.asset(nameDisable),
        onChanged = onChanged ?? onChanged,
        value = value ?? false,
        clickable = enable ?? true,
        padding = padding ?? 0;

  @override
  Widget build(BuildContext context) {
    Widget _child = value ? iconChecked : iconNormal;
    Widget _enableWidget = GestureDetector(
      onTap: () {
        value = !value;
        onChanged?.call(value);
      },
      child: _child,
    );
    return Container(
      width: size.width,
      height: size.height,
      padding: EdgeInsets.all(padding),
      child: (clickable == null || clickable) ? _enableWidget : _child,
    );
  }
}

/// 可变颜色的 Text
class CustomCheckedText extends StatelessWidget {
  final String data;
  final TextStyle? style;
  final TextAlign? textAlign;
  final TextDirection? textDirection;
  final Locale? locale;
  final bool? softWrap;
  final TextOverflow? overflow;
  final int? maxLines;
  final Color? colorCheck;
  final Color? colorNormal;
  final double? fontSize;
  bool value = false; // 选中状态
  bool? clickable = true; //是否允许点击
  final Function(bool)? onChanged; //点击监听

  CustomCheckedText(
    this.data, {
    Key? key,
    required this.value,
    this.style,
    this.textAlign,
    this.textDirection,
    this.locale,
    this.softWrap,
    this.overflow,
    this.maxLines,
    this.colorCheck,
    this.colorNormal,
    this.fontSize,
    this.clickable,
    this.onChanged,
  }) : super(key: key);

  @override
  Widget build(BuildContext context) {
    Text _text = Text(
      data,
      textAlign: textAlign ?? null,
      textDirection: textDirection ?? null,
      locale: locale ?? null,
      softWrap: softWrap ?? null,
      overflow: overflow ?? null,
      maxLines: maxLines ?? null,
      style: TextStyle(fontSize: fontSize ?? 14.0, color: value ? colorCheck : colorNormal),
    );
    GestureDetector _gestureDetector = GestureDetector(
      onTap: () {
        value = !value;
        onChanged?.call(value);
      },
      child: _text,
    );
    return (clickable == null || clickable == true) ? _gestureDetector : _text;
  }
}

class CheckTextView extends StatelessWidget {
  Widget _none = ConstrainedBox(constraints: BoxConstraints(maxWidth: 0, maxHeight: 0));
  final Widget centerWidget;
  final Widget? centerWidgetChecked;
  final Widget? drawableLeftChecked;
  final Widget? drawableLeft;
  final double? drawableLeftPadding;
  final Widget? drawableEndChecked;
  final Widget? drawableEnd;
  final double? drawableEndPadding;
  final Widget? drawableTopChecked;
  final Widget? drawableTop;
  final double? drawableTopPadding;
  final Widget? drawableBottomChecked;
  final Widget? drawableBottom;
  final double? drawableBottomPadding;
  final Function(bool)? onChanged;
  bool value = false;

  CheckTextView(
      {Key? key,
      required this.centerWidget,
      this.centerWidgetChecked,
      this.drawableLeftChecked,
      this.drawableLeft,
      this.drawableLeftPadding = 0,
      this.drawableEndChecked,
      this.drawableEnd,
      this.drawableEndPadding = 0,
      this.drawableTopChecked,
      this.drawableTop,
      this.drawableTopPadding = 0,
      this.drawableBottomChecked,
      this.drawableBottom,
      this.drawableBottomPadding = 0,
      this.value = false,
      this.onChanged})
      : super(key: key);

  @override
  Widget build(BuildContext context) {
    var mainAxisSizeRow = (drawableEndPadding == -1 || drawableLeftPadding == -1) ? MainAxisSize.max : MainAxisSize.min;
    var mainAxisSizeColumn = (drawableTopPadding == -1 || drawableBottomPadding == -1) ? MainAxisSize.max : MainAxisSize.min;
    var leftPadding = drawableLeftPadding == -1 ? Spacer() : Padding(padding: EdgeInsets.symmetric(horizontal: drawableLeftPadding ?? 0));
    var endPadding = drawableEndPadding == -1 ? Spacer() : Padding(padding: EdgeInsets.symmetric(horizontal: drawableEndPadding ?? 0));
    var topPadding = drawableTopPadding == -1 ? Spacer() : Padding(padding: EdgeInsets.symmetric(horizontal: drawableTopPadding ?? 0));
    var bottomPadding = drawableBottomPadding == -1 ? Spacer() : Padding(padding: EdgeInsets.symmetric(horizontal: drawableBottomPadding ?? 0));

    return GestureDetector(
      behavior: HitTestBehavior.translucent,
      onTap: () => onChanged?.call(value),
      child: Row(
        crossAxisAlignment: CrossAxisAlignment.center,
        mainAxisSize: mainAxisSizeRow,
        children: [
          (value ? (drawableLeftChecked ?? drawableLeft) : drawableLeft) ?? _none,
          leftPadding,
          Column(
            crossAxisAlignment: CrossAxisAlignment.center,
            mainAxisSize: mainAxisSizeColumn,
            children: [
              (value ? (drawableTopChecked ?? drawableTop) : drawableTop) ?? _none,
              topPadding,
              (value ? (centerWidgetChecked ?? centerWidget) : centerWidget),
              bottomPadding,
              (value ? (drawableBottomChecked ?? drawableBottom) : drawableBottom) ?? _none,
            ],
          ),
          endPadding,
          (value ? (drawableEndChecked ?? drawableEnd) : drawableEnd) ?? _none,
        ],
      ),
    );
  }
}

/*
class ClickableTextView extends StatelessWidget {
  final Text text;
  double width;
  double height;
  EdgeInsetsGeometry padding;
  Function() onClick; //点击监听

  ClickableTextView(this.text, {Key key, this.width, this.height, this.padding, this.onClick}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return GestureDetector(
      onTap: ()=> onClick?.call(),
      behavior: HitTestBehavior.opaque,
      child: text,
    );
  }
}
*/
