import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';

extension WidgetExt on Widget {
  /// 默认有效点击间隔一秒，interval=0 时无间隔时间
  InkWell onTap(VoidCallback onTap, {int? interval}) => _InkWell._(this, [DateTime.now().millisecondsSinceEpoch], interval, onTap);

  InkWell onLongPress(VoidCallback onLongPress) => InkWell(onLongPress: onLongPress, child: this);

  Expanded expanded({int flex = 1}) => Expanded(child: this, flex: flex);

  Padding padding(EdgeInsetsGeometry padding) => Padding(padding: padding, child: this);

  Positioned positioned({double? left, double? top, double? right, double? bottom}) => Positioned(child: this, left: left, top: top, right: right, bottom: bottom);

  SizedBox size(double width, double height) => SizedBox(width: width, height: height, child: this);

  Container container({double? width=double.infinity, double? height=double.infinity}) => Container(width: width, height: height, child: this);
}

class Debounce {
  Debounce._();

  static int _last = 0;

  static VoidCallback onTap(VoidCallback onTap, {int interval = 1000}) => () {
        var millisecondsSinceEpoch = DateTime.now().millisecondsSinceEpoch;
        if ((millisecondsSinceEpoch - _last) > interval) {
          onTap.call();
          _last = millisecondsSinceEpoch;
        }
      };
}

extension IntervalVoidCallback on VoidCallback {
  VoidCallback interval({int interval = 1000}) => Debounce.onTap(this, interval: interval);
}

class _InkWell extends InkWell {
  final Widget? child;

  _InkWell({Key? key, GestureTapCallback? onTap, this.child}) : super(key: key, child: child, onTap: onTap);

  factory _InkWell._(Widget? child, List<int>? last, int? interval, GestureTapCallback? onTap) {
    return _InkWell(
        child: child,
        onTap: () {
          var lastTime = last?[0] ?? 0;
          var millisecondsSinceEpoch = DateTime.now().millisecondsSinceEpoch;
          if ((millisecondsSinceEpoch - lastTime) > (interval ?? 1000)) {
            onTap?.call();
            last?[0] = millisecondsSinceEpoch;
          }
        });
  }
}
