import 'package:flutter/cupertino.dart';

/// 城市｜时间选择器
typedef OnCityPickerSelectChanged<T extends IPickerViewData<T>> = Function(T province, T city, T count);

class CityPicker<T extends IPickerViewData<T>> extends StatefulWidget {
  final List<T> dataList;
  final int? currentProvinceIndex; //当前所选的县区级别的index
  final int? currentCityIndex; //当前所选的县区级别的index
  final int? currentCountyIndex;
  final OnCityPickerSelectChanged<T>? selectChanged;

  const CityPicker({Key? key, required this.dataList, this.currentProvinceIndex, this.currentCityIndex, this.currentCountyIndex, this.selectChanged}) : super(key: key); //当前所选的县区级别的index

  @override
  _CityPickerState<T> createState() => _CityPickerState<T>();
}

class _CityPickerState<T extends IPickerViewData<T>> extends State<CityPicker<T>> {
  List<T> _provinceList = <T>[];
  List<T> _cityList = <T>[];
  List<T> _countyList = <T>[];
  late FixedExtentScrollController _provinceController;
  late FixedExtentScrollController _cityController;
  late FixedExtentScrollController _countyController;

  @override
  void initState() {
    List<T> dataList = widget.dataList;
    _provinceList.addAll(dataList);
    List<T> cityList = _provinceList[widget.currentProvinceIndex ?? 0].iChildBeanList;
    _cityList.addAll(cityList);
    List<T> countyList = _cityList[widget.currentCityIndex ?? 0].iChildBeanList;
      _countyList.addAll(countyList);
    _provinceController = FixedExtentScrollController(initialItem: widget.currentProvinceIndex ?? 0)
      ..addListener(() {
        _cityList.clear();
        _countyList.clear();
        List<T>? cityListTemp = _provinceList[_provinceController.selectedItem].iChildBeanList;
        if (cityListTemp != null) {
          _cityList.addAll(cityListTemp);
        }
        List<T>? countyListTemp = cityListTemp[0].iChildBeanList;
        if (countyListTemp != null) {
          _countyList.addAll(countyListTemp);
        }
        _cityController.jumpToItem(0);
        _countyController.jumpToItem(0);
        if (mounted) setState(() {});
        T provinceList = _provinceList[_provinceController.selectedItem];
        widget.selectChanged?.call(provinceList, _cityList[0], _countyList[0]);
      });
    _cityController = FixedExtentScrollController(initialItem: widget.currentCityIndex ?? 0)
      ..addListener(() {
        _countyList.clear();
        List<T> countyListTemp = _cityList[_cityController.selectedItem].iChildBeanList;
        _countyList.addAll(countyListTemp);
        _countyController.jumpToItem(0);
        if (mounted) setState(() {});
        widget.selectChanged?.call(_provinceList[_provinceController.selectedItem], _cityList[_countyController.selectedItem], _countyList[0]);
      });
    _countyController = FixedExtentScrollController(initialItem: widget.currentCountyIndex ?? 0)
      ..addListener(() {
        widget.selectChanged?.call(_provinceList[_provinceController.selectedItem], _cityList[_countyController.selectedItem], _countyList[_countyController.selectedItem]);
      });
    super.initState();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      mainAxisSize: MainAxisSize.min,
      mainAxisAlignment: MainAxisAlignment.spaceAround,
      children: [
        SizedBox(
          width: 120,
          height: 160,
          child: CupertinoPicker(itemExtent: 40, onSelectedItemChanged: (i) {}, scrollController: _provinceController, children: _provinceList.map((e) => Text(e.iDisplayText, maxLines: 1)).toList()),
        ),
        SizedBox(
          width: 120,
          height: 160,
          child: CupertinoPicker(itemExtent: 40, onSelectedItemChanged: (i) {}, scrollController: _cityController, children: _cityList.map((e) => Text(e.iDisplayText, maxLines: 1)).toList()),
        ),
        SizedBox(
          width: 120,
          height: 160,
          child: CupertinoPicker(itemExtent: 40, onSelectedItemChanged: (i) {}, scrollController: _countyController, children: _countyList.map((e) => Text(e.iDisplayText, maxLines: 1)).toList()),
        )
      ],
    );
  }
}

abstract class IPickerViewData<T extends IPickerViewData<T>> {
  String get iDisplayText;

  List<T> get iChildBeanList;
}
